"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  Builder: () => Builder,
  DTO: () => DTO,
  Property: () => Property
});
module.exports = __toCommonJS(src_exports);

// src/dto-default.decorator.ts
var import_injection = require("@autometa/injection");

// src/property.enum.ts
var PROPERTY_DEFAULTS = Symbol("autometa:meta-data:property-defaults");
var DtoBuilderSymbols = {
  PROPERTY_DEFAULTS
};

// src/dto-default.decorator.ts
var import_any_date_parser = require("any-date-parser");
var DefaultValueDecorators = {
  value: (value) => (target, propertyKey) => {
    const container = (0, import_injection.metadata)(getConstructor(target)).custom(
      DtoBuilderSymbols.PROPERTY_DEFAULTS,
      {},
      false
    );
    const datum = { value };
    container[propertyKey] = datum;
  },
  factory: (factory) => (target, propertyKey) => {
    const container = (0, import_injection.metadata)(getConstructor(target)).custom(
      DtoBuilderSymbols.PROPERTY_DEFAULTS,
      {},
      false
    );
    const datum = { factory };
    container[propertyKey] = datum;
  },
  dto: (dtoType) => (target, propertyKey) => {
    const container = (0, import_injection.metadata)(getConstructor(target)).custom(
      DtoBuilderSymbols.PROPERTY_DEFAULTS,
      {},
      false
    );
    const datum = { dtoType };
    container[propertyKey] = datum;
  },
  date: (stamp) => {
    return (target, propertyKey) => {
      const container = (0, import_injection.metadata)(getConstructor(target)).custom(
        DtoBuilderSymbols.PROPERTY_DEFAULTS,
        {},
        false
      );
      const datum = {
        factory: () => {
          if ("number" === typeof stamp) {
            return new Date(stamp);
          }
          if (!stamp) {
            return /* @__PURE__ */ new Date();
          }
          return Date.fromString(stamp);
        }
      };
      container[propertyKey] = datum;
    };
  }
};
function getConstructor(target) {
  if (target && "constructor" in target) {
    return target.constructor;
  }
  throw new Error(`Cannot get constructor for ${target}`);
}

// src/property.decorator.ts
function Property(defaultOrFactory) {
  return determineDefaultType(defaultOrFactory);
}
function determineDefaultType(defaultOrFactory) {
  if (isClass(defaultOrFactory)) {
    return DefaultValueDecorators.dto(defaultOrFactory);
  }
  if ("function" === typeof defaultOrFactory) {
    return DefaultValueDecorators.factory(defaultOrFactory);
  }
  if (defaultOrFactory instanceof Date.constructor) {
    return DefaultValueDecorators.date();
  }
  return DefaultValueDecorators.value(defaultOrFactory);
}
function isClass(value) {
  if (!("function" != typeof value)) {
    return false;
  }
  return /^class[\s{]/.test(toString.call(value));
}

// src/dto.ts
function DTOWrapper() {
  return class {
  };
}
var DTO = Object.assign(DTOWrapper, DefaultValueDecorators);

// src/interface-proxy.ts
var import_injection2 = require("@autometa/injection");
function Builder(defaults) {
  if (isClassBuilder(defaults)) {
    return class BuilderClass {
      constructor() {
        const built2 = { ...defaults };
        const proxy = classProxy.bind(null, defaults, built2);
        return proxy(this);
      }
      static fromRaw(value) {
        const builder2 = new this();
        const keys = Object.keys(value);
        for (const key of keys) {
          builder2.assign(key.toString(), value[key]);
        }
        return builder2;
      }
      static default() {
        return new this().build();
      }
    };
  }
  const built = { ...defaults };
  const builder = new Proxy(built, {
    get(target, prop) {
      if ("build" === prop) {
        return () => built;
      }
      if ("derive" === prop) {
        return () => Builder({ ...built });
      }
      if ("assign" === prop) {
        return (property, value) => {
          built[property.toString()] = value;
          return target;
        };
      }
      if ("attach" === prop) {
        return (property, subProperty, value) => {
          if (typeof built[property.toString()] === void 0) {
            built[property.toString()] = {};
          }
          const dict = built[property.toString()];
          dict[subProperty] = value;
          return target;
        };
      }
      if ("append" === prop) {
        return (property, value) => {
          if (!Array.isArray(built[property.toString()])) {
            built[property.toString()] = [value];
          } else {
            built[property.toString()].push(value);
          }
          return target;
        };
      }
      return (...args) => {
        if (0 === args.length) {
          return built[prop.toString()];
        }
        built[prop.toString()] = args[0];
        return builder;
      };
    }
  });
  return builder;
}
function classProxy(defaults, built, inst) {
  const clsBuilder = new Proxy(inst, {
    get(_, prop) {
      if ("build" === prop) {
        return () => {
          const instance = constructDefaultDto(defaults);
          return Object.assign(instance, { ...built });
        };
      }
      if ("assign" === prop) {
        const fn2 = (property, value) => {
          built[property.toString()] = value;
          return clsBuilder;
        };
        return fn2;
      }
      if ("append" === prop) {
        const fn2 = (property, value) => {
          if (!Array.isArray(built[property.toString()])) {
            built[property.toString()] = [value];
          } else {
            built[property.toString()].push(value);
          }
          return clsBuilder;
        };
        return fn2;
      }
      if ("attach" === prop) {
        return (property, subProperty, value) => {
          if (built[property.toString()] === void 0) {
            built[property.toString()] = {};
          }
          const dict = built[property.toString()];
          dict[subProperty] = value;
          return clsBuilder;
        };
      }
      if (prop in inst) {
        return inst[prop];
      }
      const fn = (...args) => {
        if (0 === args.length) {
          return built[prop.toString()];
        }
        built[prop.toString()] = args[0];
        return clsBuilder;
      };
      const modified = Object.defineProperty(fn, "value", {
        get() {
          return built[prop.toString()];
        }
      });
      return modified;
    }
  });
  return clsBuilder;
}
function isClassBuilder(cls) {
  return "function" === typeof cls;
}
function constructDefaultDto(base) {
  const defaultSetters = (0, import_injection2.metadata)(base).getCustom(
    DtoBuilderSymbols.PROPERTY_DEFAULTS
  );
  const instance = new base();
  if (!defaultSetters) {
    return instance;
  }
  const keys = Object.keys(defaultSetters);
  for (const key of keys) {
    const value = defaultSetters[key];
    if ("value" in value) {
      Object.defineProperty(instance, key, {
        value: value.value,
        writable: true,
        enumerable: true
      });
    }
    if ("factory" in value) {
      Object.defineProperty(instance, key, {
        value: value.factory(),
        writable: true,
        enumerable: true
      });
    }
    if ("dtoType" in value) {
      const dtoInst = constructDefaultDto(value.dtoType);
      Object.defineProperty(instance, key, {
        value: dtoInst,
        writable: true,
        enumerable: true
      });
    }
  }
  return instance;
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  Builder,
  DTO,
  Property
});
//# sourceMappingURL=index.js.map